"""
	ripper_dialog.py
		the dialogs for RoxCD's CD ripping/encoding.
	Copyright 2004 Ron Kuslak. <rds@rdsarts.com>
		All rights reserved.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import findrox; findrox.version(1, 9, 12)
import rox, ripper_lib, os, random, sys, re
from rox import saving, g, filer

_ = rox.i18n.translation(os.path.join(rox.app_dir, 'Messages'))

APP_NAME = 'ripper_dialog'
APP_VER  = '0.5'

# 'Constants' for translation
INFO_LABEL = _('Click "Close" to close this window and stop ripping/encoding at any time.')
DIALOG_TITLE = _('Ripping/Encoding')
RIP_LIST_TITLE = _('To rip/encode:')
FINISHED_TITLE = _('Completed:')
WAITING_TEXT = _('Waiting...')
RIP_TEXT = 'Ripping: (%s)'
ENC_TEXT = 'Encoding: (%s)'
QUEUE_TEXT = _('Queue:')
FINISHED_TEXT = _('Completed:')

def debugger(*text):
	print >>sys.stderr, '%s %s: %s'%(APP_NAME, APP_VER, text)
	return

def regexp_swap(string, search_str, re_str):
	# Swaps string for regex 're_str' in search_str
	search_str = str(search_str)
	expression = re.compile(r'%s'%re_str)
	return expression.sub(search_str, string)

def fix_cmd_constants(cmd_line, artist, album, genre, year):
	# Replaces %var% with variable for making a string ready for the ripper/encoder
	for rep_str, rex in [(artist, '%artist%'), (album, '%album%'), (genre, '%genre%'), (year, '%year%')]:
		cmd_line = regexp_swap(cmd_line, rep_str, rex)
	return cmd_line

def fix_cmd_track(cmd_line, track, track_name):
	# Replaces %var% with variable for making a string ready for the ripper/encoder
	for rep_str, rex in [(track, '%track%'), (track_name, '%track_name%')]:
		cmd_line = regexp_swap(cmd_line, rep_str, rex)
	return cmd_line

def fix_cmd_filename(cmd_line, dest_name, cd_name):
	# Changes %filename% for the filename passed.
	for rep_str, rex in [(dest_name, '%filename%'), (cd_name, '%cd_filename%')]:
		cmd_line = regexp_swap(cmd_line, rep_str, rex)
	return cmd_line

class progress_label(g.HBox):
	# A widget that creates a horizontal label, progress bar, and button combination
	
	def __init__(self, label = '', text = '', callback = None):
		g.HBox.__init__(self)
		self.label = g.Label()
		self.label_text = label
		self.progress = g.ProgressBar()
		self.button = g.Button(stock = g.STOCK_STOP)
		
		self.set_text(text)
		self.pack_start(self.label, g.FALSE, g.FALSE)
		self.pack_end(self.button, g.FALSE, g.FALSE, 4)
		self.pack_end(self.progress, g.FALSE)
		self.set_callback(callback)
		self.set_sensitive(g.FALSE)

	def set_callback(self, callback = None):
		if callback:
			self.button.connect('activate', callback)

	def set_text(self, text):
		self.label.set_text(self.label_text%text)

	def pulse(self):
		self.progress.pulse()

	def set_sensitive(self, sensitivity):
		# Sets the sensitivity of the button.
		self.button.set_sensitive(sensitivity)

	def set_progress(self, text = None, percent = None):
		if text:
			self.progress.set_text(text)
		if percent:
			self.progress.set_fraction(percent)

class track_list(g.Frame):

	def __init__(self, text = ''):
		g.Frame.__init__(self)
		self.cmds = []
		self.listview = g.ListStore(str)
		self.treeview = g.TreeView(self.listview)
		self.treeview.set_rules_hint(g.TRUE)
		self.treeview.set_headers_visible(g.FALSE)
		self.label = g.Label(text)
		sw = g.ScrolledWindow()
		sw.set_policy(g.POLICY_AUTOMATIC, g.POLICY_ALWAYS)
		col = g.TreeViewColumn(None, g.CellRendererText(), text = 0)

		self.treeview.insert_column(col, 0)

		sw.add(self.treeview)
		self.add(sw)

		# self.set_sensitive(g.FALSE)

		self.set_label_widget(self.label)
	
	def update_listview(self):
		self.listview.clear()
		if len(self.cmds) <= 0:
			self.set_sensitive(g.FALSE)
			return
		self.set_sensitive(g.TRUE)
		for title, cmd, cmd2, cd_filename in self.cmds:
			self.listview.append([title])


	def push_cmds(self, text, cmd1, cmd2, cd_filename):
		# Adds text as label and temporary filename for cmd1 and cmd2 to list.
		self.cmds.append([text, cmd1, cmd2, cd_filename])
		self.update_listview()

	def get_cmds(self, index):
		# Returns a matrix of [track name, cmd1, cmd2]
		return self.cmds[index]

	def pop_cmds(self, index = None):
		# Pulls top entry (or index) from list, and removes it from our lists.
		if not index:
			index = 0
		temp = self.get_cmds(index)
		del self.cmds[index]
		self.update_listview()
		return temp

	def number_of_tracks(self, *args):
		# Returns how many tracks we have
		return len(self.cmds)

class rip_enc_dialog(rox.Dialog):
	# A dialog to pop-up when we are ripping or encoding. Isn't that nice of it?

	rippers = []
	engine = None

	def __init__(self, threads):
		rox.Dialog.__init__(self, DIALOG_TITLE, None, 0, ((g.STOCK_CLOSE, g.RESPONSE_CANCEL)))
		pan = g.HPaned()
		image = g.Image()
		dialog_label = g.Label(INFO_LABEL)
		rip_proglabel = progress_label(RIP_TEXT, WAITING_TEXT)
		enc_proglabel = progress_label(ENC_TEXT, WAITING_TEXT)
		self.queue = track_list(QUEUE_TEXT)
		self.finished = track_list(FINISHED_TEXT)
		self.set_has_separator(g.FALSE)
		self.engine = ripper_lib.main_engine(self.queue, self.finished, rip_proglabel, enc_proglabel)

		# Puts the 'cursor' focus on the 'close' button. Yes, it's cheap, but so am I. :P :)
		self.set_default_size(-1, 350)
		self.show_all()

		# Build info bar on top of dialog
		hbox = g.HBox()
		image.set_from_stock(g.STOCK_DIALOG_INFO, g.ICON_SIZE_DIALOG)
		dialog_label.set_line_wrap(g.TRUE)
		hbox.pack_start(image, g.FALSE, g.FALSE)
		hbox.pack_start(dialog_label, g.FALSE, g.FALSE)

		self.vbox.pack_start(hbox, g.FALSE, g.FALSE)
		self.vbox.pack_start(g.HSeparator(), g.FALSE, g.FALSE)
		del hbox

		for widget in [rip_proglabel, enc_proglabel]:
			widget.set_progress(text = WAITING_TEXT)
			self.vbox.pack_start(widget, g.FALSE, g.FALSE, 2)

		pan.pack1(self.queue, g.TRUE, g.TRUE)
		pan.pack2(self.finished, g.TRUE, g.TRUE)

		self.vbox.pack_start(pan, padding = 2)

		self.vbox.show_all()

		for track, enc_cmd, rip_cmd, cd_filename in threads:
			self.queue.push_cmds(_('Track %i'%track), enc_cmd, rip_cmd, cd_filename)
			# self.engine.add_cmds(enc_cmd, rip_cmd, cd_filename, scroller)
			# XXX: Add logic to Ripper_lib, and pass reference to all labels/progressbars
			# ... or just move a lot of that logic here. That's good too.

		self.engine.start()

	def stop(self, *dummy):
		# Stops all rippers.
		self.engine.stop_end()

class savable(saving.Saveable):
	# Savable for RoxCD DnD saving of a CD/track

	rippers = []
	tracks = []
	savebox_window = None
	make_dir = False
	encoder, ripper, filename_regexp = '', '', ''

	def __init__(self, enc_cmd, rip_cmd, filename_regexp, track_list, rip_cd):
		self.tracks		= track_list
		self.encoder		= enc_cmd
		self.ripper		= rip_cmd
		self.make_dir	= rip_cd

	def set_save_box(self, savebox):
		self.savebox_window = savebox

	def is_directory(self, value):
		# Call with True to indicate we're making a whole mess'a files in one directory, instead of just a single file
		self.make_directory = value

	def save_done(self, *args):
		# Called after everything is done. Y'all.

		# I think this is done in the default class. Not needed?
		# self.savebox_window.destroy()

		# rox.toplevel_unref()
		pass 

	def save_to_file(self, path):
		path = os.path.abspath(path)
		cmd_strings = []

		if self.savebox_window:
			self.savebox_window.hide()

		for track, track_name, filename_track in self.tracks:
			if self.make_dir:
				# Saving a whole CD. Make destination directory.
				if not os.path.exists(path):
					os.makedirs(path)
				filename = os.path.abspath(path + '/' + filename_track)
				# debugger(filename)
			else:
				filename = os.path.abspath(path)

			cd_filename = os.path.abspath(filename + '.wav')

			# Prep ripper strings:
			enc_cmd = fix_cmd_filename(self.encoder, filename, cd_filename)
			enc_cmd = fix_cmd_track(enc_cmd, track, track_name)
			rip_cmd = fix_cmd_filename(self.ripper, filename, cd_filename)
			rip_cmd = fix_cmd_track(rip_cmd, track, track_name)

			cmd_strings.append([track, enc_cmd, rip_cmd, cd_filename])
		if self.savebox_window:
			self.savebox_window.hide()

		dialog = rip_enc_dialog(cmd_strings)
		dialog.run()
		dialog.stop()
		dialog.destroy()
		rox.filer.examine(path)
