#!/usr/bin/env python
import findrox, sys
from threading import Thread
findrox.version(1,9,12)
import rox
from rox import g
from cd_logic import cd_logic as cd
from PyCDDB import PyCDDB as cddb

APP_NAME = 'cddb_gui.py'
APP_VER = '0.5.0'

def debugger(*text):
	print >>sys.stderr, "%s %s: %s"%(APP_NAME, APP_VER, text)

class cddb_query(Thread):
	''' A class the provides quick, easy access to CDDB via PyCDDB. Does not check that you have
		initialized everything. :P '''
	done = False

	def __init__(self, window, label):
		Thread.__init__(self)
		self.window = window
		self.label = label
		return

	def gtk_work(self, text = None, respond = None):
		# Does all our locking and GTK work.
		if self.done:
			# Must have quit, don't bother going on.
			return

		g.threads_enter()
		try:
			if text:
				self.label.set_text(text)
			if respond:
				self.window.response(respond)
		except:
			debugger('Something went funky in gtk_work. Passing to avoid big problems...')
		g.threads_leave()

	def get_items(self):
		# Gets the local CDDB info, and tosses it to find out the real CDDB info.
		self.gtk_work('Attempting to read from CDDB server...')
		temp = cd.get_cddb_id()
		db = cddb()
		items = db.query(temp)
		return items

	def get_tracks(self, items = []):
		# Gets all tracks for cd. Returns a matrix of all hits from CDDB.
		self.gtk_work('Reading CDDB data..')
		cddb_entries = []
		db = cddb()

		# This will just be skipped if we have no items.
		for item in items:
			info = db.read(item)
			cddb_entries.append(info)

		return cddb_entries
		
	def get_info_from_items(self, items, item_number):
		# Pulls all info from item item_number in items. Convience wrapper over get_info.
		try:
			item = items[item_number]
			return self.get_info(item)
		except:
			return 'Unknown', 'Unknown', 'Unknown', 'Unknown', []
	
	def get_info(self, item):
		# Pulls all info from item. -1 to initalize.
		temp = item['DTITLE'].split('/')

		try:
			album, artist = temp[1].strip(), temp[0].strip()
		except:
			album, artist = item['DTITLE'], 'Unknown'

		year, genre = item['DYEAR'], item['DGENRE']
		return ([album, artist, year, genre, item['TTITLE']])

	def run(self):
		self.cds = self.get_tracks(self.get_items())
		self.gtk_work(respond = g.RESPONSE_OK)


class cddb_dialog(rox.Dialog):
	selected = None

	def __init__(self, cds, query):
		rox.Dialog.__init__(self)
		hbox = g.HBox()
		pane_view = g.VPaned()
		label_hbox = g.HBox()
		cdlist_list = g.ListStore(str)
		cdlist_view = g.TreeView(cdlist_list)
		tracklist_list = g.ListStore(str)
		tracklist_view = g.TreeView(tracklist_list)
		image = g.Image()
		image_label = g.Label('Multiple CDDB entries were found. Please select the one that closest matches your CD.')

		# Tracklist is a matrix of each albums tracknames. 
		# CDList is a matrix of each album by name, artist, year and genre.
		cdlist = []
		tracklist = []

		def set_selected(cd_number):
			''' Sets cdlist_view selection to row cd_number. '''

			if cd_number > len(cdlist) - 1:
				cd_number = len(cdlist) - 1
			cdlist_view.set_cursor((cd_number))
			cdlist_view.scroll_to_cell((cd_number))

		def get_selected():
			'''	Returns currently selected track from playlist.
				Does not pass go, nor collect $200. '''
			try:
				temp, iter = cdlist_view.get_selection().get_selected()
				temp = cdlist_list.get_path(iter)[0]
				return temp
			except:
				return -1

		def set_tracklist(widget):
			''' Sets the view of tracknames to the currently selected CD.
				Scrolls to that CD and selects it just to be 100% certain. ;) '''

			cd_number = get_selected()
			tracklist_list.clear()
			for track in tracklist[cd_number]:
				tracklist_list.append([track])
			self.selected = cd_number

		def set_and_return(*dummy):
			''' Called when a row in cdlist_view is selected. Does clean-ups
				and exits. '''

			self.selected = get_selected()
			self.response(g.RESPONSE_OK)

		for iter in cds:
			# get_info returns [ album, artist, year, genre, [tracks] ]
			temp = query.get_info(iter)
			cdlist.append([temp[0], temp[1], temp[2], temp[3]])
			tracklist.append(temp[4])

		# Adds each CD with Title, Artist, Year and Genre in order
		for iter in cdlist:
			cdlist_list.append(['%s by %s (%s in %s)'%(iter[0], iter[1], iter[2], iter[3])])

		# Init our columns
		for title, view in (('Track Names', tracklist_view), ('Entries Found', cdlist_view)):
			col = g.TreeViewColumn(title, g.CellRendererText(), text = 0)
			view.insert_column(col, 0)
			

		image.set_from_stock(g.STOCK_DIALOG_INFO, g.ICON_SIZE_DIALOG)
		image_label.set_line_wrap(g.TRUE)

		set_selected(0)
		set_tracklist(None)
		tracklist_view.set_rules_hint(True)
		cdlist_view.set_rules_hint(True)

		self.set_default_size(300, 400)
		self.set_has_separator(g.FALSE)
		self.set_title('Multiple CDDB Entries.')

		self.add_button(g.STOCK_OK, g.RESPONSE_OK)
		label_hbox.pack_start(image)
		label_hbox.pack_start(image_label)
		self.vbox.pack_start(label_hbox, g.FALSE, g.FALSE)
		self.vbox.pack_start(pane_view, padding = 2)
		
		# add scroll windows to VBox.
		panel_items = []
		for widget in [cdlist_view, tracklist_view]:
			frame = g.Frame()
			sw = g.ScrolledWindow()

			sw.set_policy(g.POLICY_AUTOMATIC, g.POLICY_AUTOMATIC)

			sw.add(widget)
			frame.add(sw)
			# self.vbox.pack_start(frame, padding = 2)
			panel_items.append(frame)

		pane_view.pack1(panel_items[0], g.TRUE, g.FALSE)
		pane_view.pack2(panel_items[1], g.TRUE, g.FALSE)

		cdlist_view.connect('row-activated', set_and_return)
		cdlist_view.connect('cursor-changed', set_tracklist)
		self.show_all()

def query_wrapper(device, cds = []):
	query_dialog = rox.Dialog('Network activitity', None, g.DIALOG_MODAL| g.DIALOG_NO_SEPARATOR,
				(g.STOCK_CANCEL, g.RESPONSE_CANCEL))
	query_image = g.Image()
	query_label = g.Label('')
	hbox = g.HBox()

	query_image.set_from_stock(g.STOCK_DIALOG_INFO, g.ICON_SIZE_DIALOG)
	hbox.pack_start(query_image, False, False)
	hbox.pack_start(query_label, fill = False)
	query_dialog.vbox.pack_start(hbox, False, False)
	query_dialog.set_default_size(150, 100)
	query_label.set_line_wrap(g.TRUE)
	query_dialog.show_all()

	query = cddb_query(query_dialog, query_label)
	response = None
	ret_value = []
	query.start()
	if query_dialog.run() == g.RESPONSE_OK:
		cds = query.cds

	# Clean up the dialog's stuff.
	query_dialog.destroy()
	query.done = True

	if len(cds) >= 1:
		selector = cddb_dialog(cds, query)
		response = selector.run()
		selector.destroy()
	else:
		ret_value = 'Unknown', 'Unknown', '1700', 'Unknown', []

	if response == g.RESPONSE_OK:
		ret_value = query.get_info(cds[selector.selected])

	return ret_value
