#!/usr/bin/env python
'''
	A simple CD player for the ROX desktop.
	Copyright 2003 Ron Kuslak. <rds@rdsarts.com>
	All rights reserved.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License version 2 as
	published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''

import findrox
findrox.version(1,9,14)

import os, sys, cddb_engine, widgets, rox, rename_dialog
from rox import g, Menu
from rox.options import Option
from cd_logic import cd_logic
from cd_logic.CDLow import *
from debug_utils import *
from cd_data import CdData

# Some bland 'globals'
APP_NAME = 'RoxCd'
EXPAND_FILL = g.EXPAND|g.FILL

class RoxCd(rox.Window):
	def __init__(self):
		rox.Window.__init__(self)
		self.data_init()
		self.ui_init()
		self.options_init()
		self.menu_init()

		self.update_ui()
		self.connects_init()

	def connects_init(self):
		# Adds needed connects and timeouts for core widgets.
		for widget in [self, self.artist_entry, self.album_entry, self.genre.entry, self.year_range]:
			widget.connect('popup-menu', lambda *args: self.menu_handler())
			widget.connect('button-press-event', lambda widget, event: self.menu_handler(event))

		self.connect('popup-menu', lambda *args: self.menu_handler())
		self.connect('delete-event', lambda *args: self.close_evt())
		self.connect('destroy', self.close_evt)
		self.tracklist_widget.treeview.connect('row-activated', lambda *arg: self.play_selected() )
		self.tracklist_widget.treeview.connect('button-press-event', lambda widget, event: self.menu_handler(event) )

		# Set up thread for GUI updates
		g.timeout_add(500, self.update_ui)

	def data_init(self):
		# Zero out everything the class needs here. Should only be called ONCE. EVER.
		self.cur_track = -1
		self.cd_status = None
		self.cd_data = CdData()
		self.reload_playlist = True
		self.title_text = ''
		self.cur_time = None
		self.tot_time = None

	def menu_init(self):
		# Initialize the menu, and attach it to this class' window
		Menu.set_save_name(APP_NAME)
		track_menu = Menu.SubMenu(_('_Track'), [
			Menu.Action(_('/Play Track'),		'play_selected',	None,	g.STOCK_GO_FORWARD),
			Menu.Action(_('/_Rename Track'),	'rename_track',	'F2',	g.STOCK_FIND_AND_REPLACE),
			])
		cd_menu = Menu.SubMenu(_('_Disc'), [
			Menu.Action(_('/_Play\/Pause'),		'play_pause', 		'<Ctrl>Return',	g.STOCK_GO_FORWARD),
			Menu.Action(_('/_Next'),				'next', 				'<Ctrl>Right',	g.STOCK_GOTO_LAST),
			Menu.Action(_('/_Previous'),			'prev', 				'<Ctrl>Left',		g.STOCK_GOTO_FIRST),
			Menu.Action(_('/_Stop\/Eject'),			'stop', 				'Escape',		g.STOCK_STOP),
			Menu.Action(_('/Reload from CDDB'),	'set_cd_from_cddb',	'<Ctrl>R',		g.STOCK_REFRESH)])
		rip_menu = Menu.SubMenu(_('Rip'), [
			Menu.Action(_('/Rip CD'),				'rip_cd',				'<Shift><Ctrl>S',	g.STOCK_CDROM),
			Menu.Action(_('/Rip selected track'),		'rip_track',			'<Ctrl>S',		g.STOCK_FLOPPY),
			])
		self.menu = Menu.Menu('main', [
			track_menu,
			cd_menu,
			rip_menu,
			Menu.Separator(),
			Menu.Action(_('/_Options'),				'show_options',		'<Ctrl>O',		g.STOCK_PREFERENCES),
			Menu.Action(_('/_Quit'),				'close_evt',			'<Ctrl>Q',		g.STOCK_CLOSE)
		])
		self.menu.attach(self, self)

	def ui_init(self):
		# Sets up the UI. Makes all our widgets. Etc..
		self.cd_data = CdData(APP_NAME)
		self.time_scale = widgets.TimeScale()
		self.artist_entry = widgets.Entry()
		self.album_entry = widgets.Entry()
		self.toolbar = widgets.Toolbar(self)
		self.tracklist_widget = widgets.PlayList()
		self.year_range = widgets.Year()
		self.genre = widgets.Genre()

		self.table = g.Table(2, 4, g.FALSE)
		self.table.attach(self.time_scale,				0, 2, 0, 1, EXPAND_FILL, 0,				2, 2)
		self.table.attach(self.make_top_widget_table(),	0, 2, 1, 2, EXPAND_FILL, 0,				4, 0)
		self.table.attach(self.tracklist_widget,			0, 2, 2, 3, EXPAND_FILL, EXPAND_FILL,	2, 2)

		vbox = g.VBox()
		vbox.pack_start(self.toolbar, expand = g.FALSE, fill = g.TRUE)
		vbox.pack_start(self.table)
		self.add(vbox)

		self.resize(10, 280)
		# Workaround: GTK+ 2.4 is daft. Places focus on toolbar at exposure. This fixes that.
		self.time_scale.grab_focus()

	def options_init(self):
		# Called to set up class options, and then updates all GUI elements.
		rox.setup_app_options(APP_NAME)
		self.cd_device	= Option('device', '/dev/cdrom')
		self.color_playing	= Option('color_playing', '#a318a318ffff')
		self.filename		= Option('filename', '%track% - %artist% - %track_name%')
		self.dir_name	= Option('dir_name', '%artist% - %album%')
		self.ext_name	= Option('ext_name', '.ogg')
		self.encode		= Option('encode', 'oggenc -t "%track_name%" -l "%album%" -a "%artist%" -G "%genre%" -o "%filename%" "%cd_filename%"')
		self.ripper			= Option('ripper', 'cdparanoia -e %track% "%cd_filename%"')
		rox.app_options.add_notify(self.update_options)
		rox.app_options.notify()

	def menu_handler(self, event = None):
		# Called when we have a mouse event in the playlist. Should mean we've got a right-click to play with.
		if event != None and event.button != 3:
			return g.FALSE
		self.menu.popup(self, event)
		return g.TRUE

	def make_top_widget_table(self):
		# Makes a table of all the widgets that appear above the tracklist.
		table = g.Table(2, 4, g.FALSE)
		x = 0
		for label_txt, widget in [	[_('_Artist'),	self.artist_entry],
							[_('A_lbum'),	self.album_entry],
							[_('_Year'),	self.year_range],
							[_('_Genre'),	self.genre] ]:
				temp = g.Label('')
				temp.set_text_with_mnemonic(label_txt)
				temp.set_mnemonic_widget(widget)
				temp.set_alignment(2, 1)
				table.attach(temp,	0, 1, x, x + 1,	g.FILL,			0,		4, 0)
				table.attach(widget,	1, 2, x, x + 1,	g.EXPAND|g.FILL, 	0,		0, 1)
				x += 1
		del x
		return table

	def update_options(self):
		# Called any time when a change is made to app's options.
		if self.cd_device.has_changed:
			cd_logic.set_dev(self.cd_device.value)
			cd_close()
			cd_open()
		self.tracklist_widget.set_color(self.color_playing.value)

	def show_options(self, *dummy):
		# Callback for menu/toolbar to show optionbox
		rox.edit_options()

	def set_sensitivities(self, value):
		# Sets various parts of the interface sensitive/insensitive based on the value of... value.
		self.toolbar.set_sensitive(value)
		self.table.set_sensitive(value)

	def update_sensitive(self):
		if self.cd_status == ERROR:
			self.set_sensitivities(g.FALSE)
		else:
			self.set_sensitivities(g.TRUE)
			if self.cd_status == STOPPED or self.cd_status == NOSTATUS:
				self.time_scale.set_sensitive(g.FALSE)
				self.toolbar.set_stop_icon(g.FALSE)
			else:
				self.time_scale.set_sensitive(g.TRUE)
				self.toolbar.set_stop_icon(g.TRUE)
			
			
	def update_ui(self):
		# called every few seconds to update the various infos in the GUI.
		sensitives, scale_sensitive, show_stop = True, True, True

		g.threads_enter()
		if self.update_cd_data():
			# debugger('Setting sensitives and highlight to %d'%self.cur_track)
			self.tracklist_widget.set_highlighted(self.cur_track)
			self.update_sensitive()

		if self.reload_playlist:
			self.update_playlist()
			self.tracklist_widget.set_highlighted(self.cur_track)

		# self.tracklist_widget.set_highlighted(self.cur_track)

		if self.cd_status == PLAYING:
			# We set tot_time, implying we want to set the ranges.
			self.time_scale.set_range(0, self.tot_time)
			self.time_scale.set_value(self.cur_time)

		self.set_title( self.get_title_text(self.cd_status) )
		g.threads_leave()
		return g.TRUE


	def update_cd_data(self):
		'''	Called to set class-globals, such as CD status and whatnot. 
			Call before changing visual UI elements. Returns True if
			anything has changed since last call.
		'''
		# XXX: Split this into 'update_cd_status()' and 'update_cur_track()'?
		try:
			temp = get_status()
			temp_track = cd_logic.current_track()
			# XXX: These cause something funky to happen if you play with the options
			# while a cd is playing. Investigate before 0.6, hopefully we be fixed with
			# CDLow rewrite.
			self.cur_time = cd_logic.get_track_time()
			self.tot_time = cd_logic.get_track_time_total()
		except UnboundLocalError, msg:
			temp = ERROR
			temp_track = 0
			self.cur_time = 0
			self.tot_time = 1
		except IOError:
			# debugger('Got IO error.')
			# debugger(temp)
			self.cur_time = 0
			self.tot_time = 1

		#debugger("temp = %s\nself.cd_status = %s"%( str(temp), str(self.cd_status) ))
		if not self.get_is_playing(temp):
			# No disc action. Set track to '0' to indicate stopped state and put highlight
			# at top of PlayList.
			temp_track = 0

		if temp == self.cd_status and temp_track == self.cur_track:
			# Everything's the same, just return.
			return False

		if self.cd_status == None or (self.cd_status != temp and temp != PAUSED):
			# status changed to something other then PAUSED, or we have no 
			# debugger('set reload_playlist')
			self.reload_playlist = True

		self.cd_status = temp
		self.cur_track = temp_track
		return True

	def get_title_text(self, status):
		# sets self.title_text, so we can set the window title to it.
		if status == ERROR:
			# This should raise a dialog. Raise a exection for calling process?
			try:
				# BUGBUGBUGBUGBUGBUGBUGBUG: Temporary fix. 
				#	Seem to always get a device error when there's not disc in the drive.
				cd_logic.check_dev()
				return self.get_title_text(NODISC)
			except:
				title_text = _("Can't open device!")
		elif status == NODISC:
			title_text = _("No CD")
		elif status == PAUSED:
			title_text = _("Paused")
		elif status == STOPPED or status == NOSTATUS:
			title_text = _("Stopped")
		else:
			title_text = _("Playing %s")%self.get_track_name(self.cur_track)
		return title_text

	def get_is_stopped(self, status = None):
		# Returns True if status (self.cd_status in not defined) is stopped.
		if status == None:
			status = self.cd_status
		if status == STOPPED or status == NOSTATUS:
			return True
		return False

	def get_is_playing(self, status = None):
		# Returns True if status is playing, False if not.
		if status == None:
			status = self.cd_status
		if status == PLAYING or status == PAUSED:
			return True
		return False

	def toolbar_play(self, *args):
		# Wrapper for the toolbar 'play' button. Yes, needing this sucks.
		self.play_pause()

	def play_selected(self, *dummy):
		# Plays track currently selected in playlist.
		track = self.tracklist_widget.get_selected()
		self.play_pause(track)

	def play_pause(self, track = None):
		# wrapper for play and pause. Plays if not playing or paused, pauses if playing.
		try:
			if track == None:
				if self.cd_status in [PLAYING, PAUSED]:
					cd_logic.pause()
				else:
					cd_logic.play(1)
				return True
			cd_logic.play(track + 1)
			return True
		except:
			debugger('Error in play_pause')
			return False

	def stop(self, *args):
		# Stops the CD playing. If none is playing, CD is ejected.
		try:
			if self.get_is_stopped():
				cd_logic.eject()
			else:
				cd_logic.stop()
		except IOError, err:
			return False
		return True

	def next(self, *args):
		# Advances CD one track, or puts it at track 1 if at last.
		cd_logic.next()
		return 0

	def prev(self, *args):
		# Tells the CD to begin go back to the  previous track, or begin playing at the last track.
		cd_logic.prev()
		return 0

	def get_track_name(self, track_number):
		# Returns the name of the track number passed. Does it's own sanity checking.
		try:
			return self.tracklist_widget.get_track_name(track_number)
		except:
			debugger('Attempted to read track name outside of tracklist size. Index #%s'%str(track_number - 1))
			return 'ERROR'

	def get_tracks(self):
		# Gets the total number of tracks in the current CD.
		try:
			cd_logic.check_dev()
			tracks = cd_logic.total_tracks()
		except IOError: 
			tracks = 0
		return tracks

	def rename_track(self):
		dialog = rename_dialog.RenameDialog(self, self.cd_data, self.tracklist_widget)
		dialog.show_all()

	def set_ui_values(self):
		# Sets the UI's data to the contents of the CdData object
		cd_d = self.cd_data
		self.artist_entry.set_text( cd_d.get_artist() )
		self.album_entry.set_text( cd_d.get_album() )
		self.genre.entry.set_text( cd_d.get_genre() )
		self.year_range.set_value( float(cd_d.get_year()) )
		self.tracklist_widget.set_tracks( cd_d.get_tracklist() )
	
	def sync_values(self):
		# Syncs the data in the UI and CdData.
		self.cd_data.set_album( self.album_entry.get_text() )
		self.cd_data.set_artist( self.artist_entry.get_text() )
		self.cd_data.set_year( self.year_range.get_value() )
		self.cd_data.set_genre( self.genre.entry.get_text() )

	def load_cddb(self, *dummy):
		# Loads tracklist from remote CDDB server.
		cd_data = None
		temp = cddb_engine.query_wrapper(self.cd_device.value)
		self.set_sensitivities(g.TRUE)
		if temp != []:
			# User selected a entry from the CDDB. Have at you.
			cd_data = CdData()
			# debugger('Setting from CDDB. Tracklist:', temp[4])
			cd_data.set_album(temp[0])
			cd_data.set_artist(temp[1])
			cd_data.set_year(temp[2])
			cd_data.set_genre(temp[3])
			cd_data.set_tracklist(temp[4])
			cd_data.set_number_of_tracks( self.get_tracks() )
			cd_data.set_cd_number( cd_logic.get_disc_id() )
		return cd_data
	
	def get_have_cd(self):
		# Returns True if we have a CD, False otherwise.
		if self.cd_status in [ERROR, NODISC]:
			return False
		return True

	def set_cd_from_cddb(self):
		# Attempts to set UI's values from a CDDB query. If user cancels, shouldn't touch anything.
		# debugger('trying to set via CDDB...')
		if self.get_have_cd():
			temp = self.load_cddb()
			if temp:
				debugger(temp)
				self.cd_data = temp
			self.set_ui_values()

	def update_playlist(self):
		# Updates listview of the playlist and other GUI elements with current data

		# debugger('updating playlist..')
		self.reload_playlist = False
		have_cd = self.get_have_cd()

		if have_cd:
			# Needs to be two layers of logic to avoid exception from get_cd_number if we have no CD.
			if cd_logic.get_disc_id() == self.cd_data.get_cd_number():
				return
		self.save_cd()

		self.cd_data = CdData()
		if have_cd:
			self.cd_data.set_cd_number( cd_logic.get_disc_id() )
			self.cd_data.set_number_of_tracks( self.get_tracks() )
		self.set_ui_values()

		if self.cd_data.load():
			self.set_ui_values()
			return
		self.set_cd_from_cddb()

	def save_cd(self):
		# debugger('Check if we should save, then save.')
		if self.cd_data.get_cd_number():
			# Save CD before we change it. ;)
			self.sync_values()
			self.cd_data.save()
			return True
		return False

	def close_evt(self, event=None):
		# Called before destroying the main RoxCD window
		self.save_cd()
		self.destroy()

	def rip(self, track_list):
		base_dirname = self.dir_name.value
		base_filename = self.filename.value
		ext = self.ext_name.value
		
		if self.cd_status in [ERROR, NODISC]:
			# no CD, or it's unavailable to us...
			return

		if not track_list or track_list == [-1]:
			# passed a bad tracklist
			return

		fix_track_list = []
		for track in track_list:
			pass

	def rip_cd(self):
		temp = []
		for iter in range(1, self.get_tracks() + 1):
			temp.append(iter)
		self.rip(temp)

	def rip_track(self):
		temp = self.tracklist_widget.get_selected()
		self.rip([temp])

if __name__ == '__main__':
	# Set up reading of translations.
	__builtins__._ = rox.i18n.translation(os.path.join(rox.app_dir, 'Messages'))

	try:
		g.threads_init()
	except:
		rox.croak(
			 _("Sorry, but the version of PyGTK currently on your computer does not support threading. Please check with your distributors, as they will most likely have a version available now that supports threaded PyGTK applications.")
		)

	main = RoxCd()
	main.show_all()

	rox.mainloop()